@@ -6,9 +6,6 @@ use IO::Socket;
 use IO::Select;
 use utf8;
 use Encode;
-use Config;
-if ($Config{usethreads}) { use threads; }
-use threads::shared;
 use vars qw($VERSION $DEBUG);
 
 use constant {
@@ -17,7 +14,7 @@ use constant {
 	DEFAULT_FLAGS        => 0,
 };
 
-$VERSION = '1.05';
+$VERSION = '1.06';
 
 use constant STREAM_BUFFER_LENGTH => 65536;
 use constant MACKET_HEADER_LENGTH => 4;
@@ -29,6 +26,15 @@ our @EXPORT = qw(DATA_TEXT DATA_BINARY);
 BEGIN {
 	package Net::Wire10;
 
+	# Use threads if available, always use threads::shared.
+	use Config;
+	if ($Config{useithreads}) {
+		require threads;
+		threads->import;
+	}
+	require threads::shared;
+	threads::shared->import;
+
 	# Macket (MySQL messages) types.
 	our %MACKET_NAMES = (
 		   1 => 'HANDSHAKE',
@@ -75,6 +81,7 @@ BEGIN {
 		"\x1A" => 'STMT_RESET',
 		"\x1B" => 'SET_OPTION',
 		"\x1C" => 'STMT_FETCH',
+		"\x1D" => 'DAEMON',
 	);
 
 	# Per-connection flags, some of which are sent to the
@@ -145,13 +152,56 @@ BEGIN {
 		0x00000400 => 'METADATA_CHANGED',
 	);
 
+	our %COLUMN_TYPES = (
+		0 => 'DECIMAL',
+		1 => 'TINY',
+		2 => 'SHORT',
+		3 => 'LONG',
+		4 => 'FLOAT',
+		5 => 'DOUBLE5',
+		6 => 'NULL',
+		7 => 'TIMESTAMP',
+		8 => 'LONGLONG',
+		9 => 'INT24',
+		10 => 'DATE',
+		11 => 'TIME',
+		12 => 'DATETIME',
+		13 => 'YEAR',
+		14 => 'NEWDATE',
+		15 => 'VARCHAR',
+		16 => 'BIT',
+		246 => 'NEWDECIMAL',
+		247 => 'ENUM',
+		248 => 'SET',
+		249 => 'TINY_BLOB',
+		250 => 'MEDIUM_BLOB',
+		251 => 'LONG_BLOB',
+		252 => 'BLOB',
+		253 => 'VAR_STRING',
+		254 => 'STRING',
+		255 => 'GEOMETRY',
+	);
+
+	our %COLUMN_FLAGS = (
+		0x000001 => 'NOT_NULL',           # Column has a NOT NULL constraint
+		# ... Skip flags that provide incomplete and/or ambiguous information under nearly all circumstances ...
+		# ... Skip deprecated flags ...
+		0x000020 => 'UNSIGNED',           # Column data type is unsigned
+		0x000040 => 'ZEROFILL',           # Column was created with the ZEROFILL flag
+		# ... Skip deprecated flags ...
+		0x000200 => 'AUTO_INCREMENT',     # Default field values for this column are generated by a thread-safe sequencer
+		# ... Skip deprecated flags ...
+		0x001000 => 'NO_DEFAULT_VALUE',   # Column does not have a default value
+		0x002000 => 'ON_UPDATE_NOW',      # Column default value is NOW() when an UPDATE affects the column
+		# ... Skip deprecated flags, internal and temporary flags ...
+	);
+
 	# Enumerations that are currently not included here:
 	#  * Server capabilities
 	#  * Server language
 	#  * Server error code
 	#  * Server error state
-	#  * SQL data types
-	#  * Column flags
+	#  * Column flags (complete)
 	#  * Collations (complete)
 
 	# TODO: An enumeration with a 2xxx error code for each client-side error.
@@ -193,6 +243,7 @@ BEGIN {
 		local $SIG{ALRM} = sub { $alarm_bug = 0; };
 		alarm 1;
 		for (my $i = 0; $i < 20; $i++) {
+			last if $alarm_bug == 0;
 			select(undef, undef, undef, 0.1);
 		}
 		local $SIG{ALRM} = sub { };
@@ -200,8 +251,25 @@ BEGIN {
 		return $alarm_bug;
 	}
 
+	sub _flags_included {
+		my $flags = shift;
+		my $i = 0;
+		while ((my $key, my $value) = each(%$flags)) {
+			$i |= $key;
+		}
+		return $i;
+	}
+
 	_assign("MAX_UINT_SIZE", _find_largest_uint);
-	_assign("HAS_ALARM_BUG", _detect_alarm_bug);
+	_assign("USEFUL_COLUMN_FLAGS", _flags_included(\%COLUMN_FLAGS));
+
+	# Pick your poison..  Detecting whether the current platform has
+	# this bug takes 1-2 seconds if the bug is present; always
+	# activating the workaround introduces an (as yet) unmeasured
+	# delay on platforms where the bug is not present.
+	#
+	#_assign("FIX_ALARM_BUG", _detect_alarm_bug);
+	_assign("FIX_ALARM_BUG", 1);
 
 	_make_constant(\%MACKET_NAMES, "MACKET_");
 	_make_constant(\%COMMAND_NAMES, "COMMAND_");
@@ -209,6 +277,8 @@ BEGIN {
 	_make_constant(\%UTF8_COLLATIONS, "UTF8_");
 	_make_constant(\%DATA_TYPES, "DATA_");
 	_make_constant(\%SERVER_STATUS, "STATUS_");
+	_make_constant(\%COLUMN_TYPES, "TYPE_");
+	_make_constant(\%COLUMN_FLAGS, "COLUMN_");
 }
 
 my %MACKET_NAMES = %{*MACKET_NAMES};
@@ -217,6 +287,8 @@ my %FLAG_NAMES = %{*FLAG_NAMES};
 my %UTF8_COLLATIONS = %{*UTF8_COLLATIONS};
 my %DATA_TYPES = %{*DATA_TYPES};
 my %SERVER_STATUS = %{*SERVER_STATUS};
+my %COLUMN_TYPES = %{*COLUMN_TYPES};
+my %COLUMN_FLAGS = %{*COLUMN_FLAGS};
 
 # Constructor
 sub new {
@@ -434,14 +506,14 @@ sub _receive_packet_data {
 	my $data;
 
 	while ($self->_check_time_remaining) {
-		# Work around a bug in alarm().
-		sleep 0 if HAS_ALARM_BUG;
 		# Cancel if requested.
 		$self->_fatal_error("Query cancelled") if $self->{cancelling};
 		# Ask every second if there is data to be read.
 		my $ready = $io_select->can_read(TIMEOUT_GRANULARITY);
 		# IO::Select sometimes returns undef instead of an empty array.
 		last if defined($ready) && (scalar($ready) != 0);
+		# Work around a bug in alarm().
+		sleep 0 if FIX_ALARM_BUG;
 	}
 
 	$socket->recv($data, STREAM_BUFFER_LENGTH, 0);
@@ -760,6 +832,7 @@ sub _send_login_message {
 		# Unsupported:  RESERVED
 		FLAG_SECURE_CONNECTION +
 		# Unsupported:  MULTI_STATEMENTS
+		# Necessary to avoid server abort:  FLAG_MULTI_RESULTS
 		FLAG_MULTI_RESULTS;
 		# Unsupported:  SSL_VERIFY_SERVER_CERT
 		# Unsupported:  REMEMBER_OPTIONS
@@ -1092,7 +1165,7 @@ sub _parse_eof_macket {
 	# Server status
 	my $status = Net::Wire10::Util::decode_my_uint($macket->{buf}, \$pos, 2);
 	printf "  -> Server status flags: 0x%x\n", $status if $self->{debug} & 1;
-	$self->_fatal_error("Query yielded multiple results") if $status & STATUS_MORE_RESULTS_EXISTS;
+	$self->_fatal_error("Query yielded multiple results (not supported)") if $status & STATUS_MORE_RESULTS_EXISTS;
 	$iterator->{warnings} = $warnings if defined($iterator);
 }
 
@@ -1128,12 +1201,15 @@ sub _parse_column_info_macket {
 	$self->_fatal_error("Truncated column info macket") if $left < 10;
 	# Charset number
 	my $collation = Net::Wire10::Util::decode_my_uint($macket->{buf}, \$pos, 2);
-	# Length
-	my $display_length = Net::Wire10::Util::decode_my_uint($macket->{buf}, \$pos, 4);
+	# Skip reading display length, because some server functions, eg. ASBINARY(),
+	# sets this to 0xFFFFFFFF unsigned, which is too big for 32-bit Perl.
+	$pos += 4;
+	#my $display_length = Net::Wire10::Util::decode_my_uint($macket->{buf}, \$pos, 4);
 	# Column data type
 	my $data_type = Net::Wire10::Util::decode_my_uint($macket->{buf}, \$pos, 1);
 	# Flags
 	my $flags = Net::Wire10::Util::decode_my_uint($macket->{buf}, \$pos, 2);
+	my $sane_flags = $flags & USEFUL_COLUMN_FLAGS;
 	# Decimal scale (for DECIMAL or NUMERIC data types)
 	my $decimal_scale = Net::Wire10::Util::decode_my_uint($macket->{buf}, \$pos, 1);
 	# Filler
@@ -1149,12 +1225,12 @@ sub _parse_column_info_macket {
 		"  -> Catalog (ignored):  %s\n  -> Name:               %s\n  -> Object:             %s\n".
 		"  -> Source Column:      %s\n  -> Source Table:       %s\n  -> Source Database:    %s\n".
 		"  -> Collation:          %s\n  -> Data Type:          %s\n  -> Decimal Scale:      %s\n".
-		"  -> Display (ignored):  %u\n  -> Flags (ignored):    0x%x\n";
+		"  -> Display (ignored):  Undeciphered\n  -> Flags:              0x%x\n";
 	printf $format,
 		$catalog, $column, $table,
 		$orig_column, $orig_table, $db,
 		$collation, $data_type, $decimal_scale,
-		$display_length, $flags
+		$flags
 	if $self->{debug} & 1;
 	return {
 		name => $column,
@@ -1164,7 +1240,8 @@ sub _parse_column_info_macket {
 		orig_column => $orig_column,
 		collation => $collation,
 		data_type => $data_type,
-		decimal_scale => $decimal_scale
+		decimal_scale => $decimal_scale,
+		flags => $sane_flags
 	};
 }
 
@@ -1885,17 +1962,32 @@ sub quote_wildcards {
 
 package Net::Wire10::Password;
 
-# Note: Digest::SHA1 is used rather than Digest::SHA because the former was
-#       in my tests slightly faster when processing small amounts of data
-#       (such as passwords).
+#
 # Note: Neither Digest::SHA nor Digest::SHA1 are pure perl modules.
-#       Digest::SHA::PurePerl exists, but is somewhat slow, so switching to
-#       that depends on a good connection pooling mechanism being added first,
-#       or everybody switching to a faster Perl interpreter (pugs/rakudo?).
+#       Digest::SHA::PurePerl exists, but is somewhat slow.
+#
+# TODO: Make use of  if Digest::SHA1 is not available.
 
 use strict;
 use warnings;
-use Digest::SHA1;
+
+BEGIN {
+	package Net::Wire10::Password;
+
+	# Use C implementation of SHA if available (currently much faster).
+	eval {
+		# Note: Digest::SHA1 is used rather than Digest::SHA because the
+		#       former was in my tests slightly faster when processing
+		#       small amounts of data (such as passwords).
+		require Digest::SHA1;
+		Digest::SHA1->import("sha1");
+	};
+	if ($@) {
+		# Otherwise fall back to the PurePerl implementation of SHA.
+		require Digest::SHA::PurePerl;
+		Digest::SHA::PurePerl->import("sha1");
+	}
+}
 
 sub scramble {
 	my $class = shift;
@@ -1910,16 +2002,10 @@ sub _make_scrambled_password {
 	my $hash_seed = shift;
 	my $password = shift;
 
-	my $ctx = Digest::SHA1->new;
-	$ctx->add($password);
-	my $stage1 = $ctx->digest;
+	my $stage1 = sha1($password);
+	my $stage2 = sha1($stage1);
+	my $result = sha1($hash_seed . $stage2);
 
-	$ctx->add($stage1);
-	my $stage2 = $ctx->digest;
-
-	$ctx->add($hash_seed);
-	$ctx->add($stage2);
-	my $result = $ctx->digest;
 	return _my_crypt($result, $stage1);
 }
 
@@ -2093,7 +2179,8 @@ Net::Wire10 implements the low-level network protocol, alias the MySQL wire prot
     printf
       "Id: %s, Message: %s\n",
       $row->{id},
-      $row->{message};
+      $row->{message}
+	;
   }
 
   $wire->disconnect;
@@ -2297,7 +2384,7 @@ Cancels the running query.  Safe to call asynchronously (thread safe).
   })->detach;
 
   # run query for 10 seconds (will be interrupted by above thread after 2 sec)
-  query("SELECT SLEEP(10)");
+  $wire->query("SELECT SLEEP(10)");
 
 Run the example above to see how cancel() works to interrupt the SLEEP(10) statement before it finishes, after only 2 seconds instead of the full 10.
 
@@ -2442,6 +2529,7 @@ The available metadata is:
   * collation (either binary, or utf8 with a given sort order and case sensitivity)
   * data_type (SQL data type)
   * decimal_scale (number of digits after the decimal point)
+  * flags (SQL data type specification and other items)
 
 =head3 has_results
 
@@ -2605,26 +2693,10 @@ Here's an example of how to retrieve warnings generated by the server.
 
 =head2 Supported operating system and Perl versions
 
-This module has been tested on these platforms.
-
-=over 4
-
-=item * Windows Server 2008
-
-with ActivePerl 5.10.0 build 1004, 32 and 64-bit
-
-=back
-
-The build script dependencies do not contain any version numbers, because
-nobody has any clue what the minimum requirements are for using this package.
-It is very recommendable, however, not to mix and match versions of C<Net::Wire10>
-and C<DBD::Wire10>.  For best results use the newest version of both packages.
-
 Over at CPAN Testers, there's a vast number of testers that do
 a very good job of figuring out which versions work together:
-L<http://static.cpantesters.org/distro/N/Net-Wire10.html>
 
-Feel free to send in reports of success or failure using different platforms.
+L<http://static.cpantesters.org/distro/N/Net-Wire10.html>
 
 =head2 Unsupported features
 
@@ -2876,16 +2948,20 @@ LAST_INSERT_ID() returns the sequence number as a string instead of as an intege
 
 There is a bug tracker at CPAN where you can report new bugs: L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Net-Wire10>
 
-Bugs in the design of the over-the-wire protocol may affect how the driver works. For example:
+Bugs in the design of the over-the-wire protocol or the server may affect how the driver works. For example:
 
 =over 4
 
 =item * No character set info is sent with metadata such as column names, forcing the driver to assume that the client and result character sets are constant, when in fact they are not.
 
-=item * Selecting data larger than max_packet_size server variable causes silent truncation (inserting does not).
+=item * Selecting data larger than max_packet_size server variable causes silent truncation on the server side (inserting does not).
+
+=item * Using tokens for LIMITs in prepared statements fails because the server chokes on quoted integers after LIMIT.
 
 =back
 
+As a workaround for the last issue, this driver scans all values given as parameters to prepared statements and skips quoting integer values.
+
 Refer to the MySQL issue tracker for more server and protocol issues: L<http://bugs.mysql.com/>
 
 Bugs and design issues in Perl itself may also affect the driver.  For example:
@@ -2894,8 +2970,12 @@ Bugs and design issues in Perl itself may also affect the driver.  For example:
 
 =item * Perl integer scalars are always signed, unlike MySQL integers.
 
+=item * Perl select() interferes with active alarm().
+
 =back
 
+A workaround is in place for the last issue, see the source code for details.
+
 Refer to the Perl issue tracker for any core Perl issues: L<http://rt.perl.org/rt3/Public/>
 
 Bugs in dependencies, for example L<IO::Socket> or L<IO::Select>, may also affect the driver.  Refer to the CPAN issue tracker for each package.
